import React, { useState } from "react";
const FontStrokeOptions = ({
  canvas,
  setopenSideBarSpacing,
  setopenSideBarFontFamily,
  openSideBarStroke,
  setopenSideBarStroke,
}) => {
  // Range selector for stroke width
  const [value, setValue] = React.useState([0]);
  // Range selector for stroke shadow
  const [shadowIsEnabled, setshadowIsEnabled] = React.useState(false);
  const [shadowoffsetValue, setshadowoffsetValue] = React.useState([0]);
  const [xoffset, setxoffset] = useState(0);
  const [yoffset, setyoffset] = useState(0);
  const [shadowBlurValue, setshadowBlurValue] = React.useState([0]);
  const [shadowColorValue, setshadowColorValue] = React.useState("black");
  // Other states
  const [strokeIsEnabled, setstrokeIsEnabled] = React.useState(false);
  const [BackgroundColor, setBackgroundColor] = React.useState("black");

  let strokeIcon = (
    <svg
      viewBox="0 0 16 16"
      fill="none"
      xmlns="http://www.w3.org/2000/svg"
      id="Border-Width--Streamline-Radix"
      height="16"
      width="16"
    >
      <desc>Border Width Streamline Icon: https://streamlinehq.com</desc>
      <path
        fill-rule="evenodd"
        clip-rule="evenodd"
        d="M0.16 2.12H15.84V3.3261866666666666H0.16V2.12ZM0.16 5.738453333333334H15.84V8.150826666666667H0.16V5.738453333333334ZM15.84 10.864640000000001H0.16V13.879999999999999H15.84V10.864640000000001Z"
        fill="currentColor"
        stroke-width="1"
      ></path>
    </svg>
  );
  let closeIcon = (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      fill="none"
      viewBox="0 0 24 24"
      stroke-width="1.5"
      stroke="currentColor"
      className="size-6"
    >
      <path
        stroke-linecap="round"
        stroke-linejoin="round"
        d="M6 18 18 6M6 6l12 12"
      />
    </svg>
  );

  const openSideBarStrokeHandler = () => {
    setopenSideBarStroke(!openSideBarStroke);
    setopenSideBarSpacing(false);
    setopenSideBarFontFamily(false);
  };
  const handleChange = (e) => {
    setValue(e.target.value); // Update the value when the slider is moved
    let ActiveObject = canvas.getActiveObject();
    if (ActiveObject && ActiveObject.type === "i-text") {
      ActiveObject.set("strokeWidth", e.target.value);
      canvas.renderAll();
    }
  };

  const strokeHandler = () => {
    setstrokeIsEnabled(!strokeIsEnabled);
    let ActiveObject = canvas.getActiveObject();
    if (!strokeIsEnabled && ActiveObject && ActiveObject.type === "i-text") {
      ActiveObject.set("stroke", true);
      canvas.renderAll();
    } else if (
      strokeIsEnabled &&
      ActiveObject &&
      ActiveObject.type === "i-text"
    ) {
      ActiveObject.set("stroke", null);
      canvas.renderAll();
    }
  };

  //   Checkbox
  const textShadowHandler = () => {
    setshadowIsEnabled(!shadowIsEnabled);
    let ActiveObject = canvas.getActiveObject();
    if (!shadowIsEnabled && ActiveObject && ActiveObject.type === "i-text") {
      ActiveObject.set("shadow", {
        color: "rgba(0,0,0,0.6)",
        blur: 10,
        offsetX: 5,
        offsetY: 5,
      });
      canvas.renderAll();
    } else if (
      shadowIsEnabled &&
      ActiveObject &&
      ActiveObject.type === "i-text"
    ) {
      ActiveObject.set("shadow", null);
      canvas.renderAll();
    }
  };

  //   Shadow direction
  // This function will set the shadow direction based on the slider value
  const getShadowDirection = (value) => {
    // Default offsets for value 0
    value = Number(value);
    if (value === 0) {
      setxoffset(0);
      setyoffset(0);
      return {
        offsetX: 0, // No horizontal offset
        offsetY: 0, // No vertical offset
      };
    }
    // Map the value (1-100) to an angle (0-360 degrees)
    const angle = (value / 100) * 2 * Math.PI; // Convert to radians
    const radius = 5; // Radius of the circular shadow movement
    let Xoffset = radius * Math.cos(angle);
    let yoffset = radius * Math.sin(angle);
    // Set the calculated offsets
    setxoffset(Xoffset);
    setyoffset(yoffset);
    return {
      offsetX: Xoffset, // Horizontal offset
      offsetY: yoffset, // Vertical offset
    };
  };

  //   Range
  const handleShadowOffset = (e) => {
    setshadowoffsetValue(e.target.value);
    let { offsetX, offsetY } = getShadowDirection(e.target.value);
    let ActiveObject = canvas.getActiveObject();
    if (ActiveObject && ActiveObject.type === "i-text") {
      // we change them dynamically
      ActiveObject.set("shadow", {
        color: shadowColorValue,
        blur: shadowBlurValue,
        offsetX,
        offsetY,
      });
      canvas.renderAll();
    }
  };
  // 3. Blur effect functions
  //   Range
  const handleBlurOffset = (e) => {
    setshadowBlurValue(e.target.value);
    let ActiveObject = canvas.getActiveObject();
    if (ActiveObject && ActiveObject.type === "i-text") {
      // We change them dynamically
      ActiveObject.set("shadow", {
        color: shadowColorValue,
        blur: e.target.value,
        offsetX: xoffset,
        offsetY: yoffset,
      });
      canvas.renderAll();
    }
  };

  //   Change Shadow color Value
  const handleShadowColor = (e) => {
    setshadowColorValue(e.target.value);
    let ActiveObject = canvas.getActiveObject();
    if (ActiveObject && ActiveObject.type === "i-text") {
      // We change them dynamically
      ActiveObject.set("shadow", {
        color: e.target.value,
        blur: shadowBlurValue,
        offsetX: xoffset,
        offsetY: yoffset,
      });
      canvas.renderAll();
    }
  };
  // Set Background color

  const handleBGColor = (e) => {
    setBackgroundColor(e.target.value);
    let ActiveObject = canvas.getActiveObject();
    if (ActiveObject && ActiveObject.type === "i-text") {
      ActiveObject.set("backgroundColor", e.target.value);
      canvas.renderAll();
    }
  };
  return (
    <div>
      <button
        onClick={openSideBarStrokeHandler}
        className="hover:bg-gray-100 focus:bg-gray-200 p-2 rounded-lg"
      >
        {strokeIcon}
      </button>
      {/* Sidebar contains all styles for Fonts */}
      {openSideBarStroke && (
        <div className="fixed top-12 left-14 bg-white shadow w-80 text-center p-2 rounded-md overflow-y-scroll">
          <div className="relative">
            <button
              onClick={() => {
                setopenSideBarStroke(false);
              }}
              className="absolute top-0 right-0"
            >
              {closeIcon}
            </button>
          </div>
          <h1 className="text-lg font-bold">Text Effects</h1>
          <div className="grid grid-cols-1 my-2 ">
            {/* Enable stroke */}
            <div
              style={{ padding: "20px", width: "300px", position: "relative" }}
              className="space-y-3"
            >
              <div className="flex justify-start items-center space-x-2">
                {/* 1.Stroke handler input */}
                <input
                  type="checkbox"
                  id="checkbox"
                  checked={strokeIsEnabled}
                  onChange={() => {
                    strokeHandler();
                  }}
                />
                <label htmlFor="checkbox">Enable Stroke</label>
              </div>
              <div className="flex justify-center items-center space-x-5">
                <input
                  type="range"
                  min="0"
                  max="15"
                  value={value}
                  onChange={handleChange}
                  disabled={strokeIsEnabled ? false : true}
                  style={{
                    width: "100%",
                    appearance: strokeIsEnabled ? "" : "none", // Remove default styling
                    background: "#ddd", // Background color of the track
                    height: "5px", // Height of the track
                    borderRadius: "5px", // Round edges for the track
                    outline: "none", // Remove outline
                  }}
                />
                <div
                  style={{
                    top: "30px",
                    padding: "3px 7px",
                    borderRadius: "4px",
                    whiteSpace: "nowrap",
                    pointerEvents: "none",
                    fontSize: "small",
                  }}
                >
                  {value}
                </div>
              </div>
              {/* 2.TextShadow  */}
              <div className="flex justify-between items-center space-x-2">
                <div className="flex justify-between items-center space-x-2">
                  {/* Checkbox Shadow */}
                  <input
                    type="checkbox"
                    id="checkbox_shadow"
                    checked={shadowIsEnabled}
                    onChange={() => {
                      textShadowHandler();
                    }}
                  />

                  <label htmlFor="checkbox_shadow">Enable Shadow</label>
                </div>
                {/* Shadow color pallete */}

                <input
                  className="w-6 h-6 cursor-pointer rounded-color-input"
                  type="color"
                  value={shadowColorValue}
                  onChange={(e) => handleShadowColor(e)}
                  disabled={shadowIsEnabled ? false : true}
                />
              </div>

              <div className="flex justify-center items-center space-x-5">
                {/* Change offset shadow left */}
                <label htmlFor="OffsetShadow">Direction</label>
                <input
                  id="OffsetShadow"
                  type="range"
                  min="0"
                  max="100"
                  step="1"
                  value={shadowoffsetValue}
                  onChange={handleShadowOffset}
                  disabled={shadowIsEnabled ? false : true}
                  style={{
                    width: "100%",
                    appearance: shadowIsEnabled ? "" : "none", // Remove default styling
                    background: "#ddd", // Background color of the track
                    height: "5px", // Height of the track
                    borderRadius: "5px", // Round edges for the track
                    outline: "none", // Remove outline
                  }}
                />
                <div
                  style={{
                    top: "30px",
                    padding: "3px 7px",
                    borderRadius: "4px",
                    whiteSpace: "nowrap",
                    pointerEvents: "none",
                    fontSize: "small",
                  }}
                >
                  {shadowoffsetValue}
                </div>
              </div>

              {/* 3. Blur text shadow */}

              <div className="flex justify-center items-center space-x-5">
                {/* Change offset shadow */}
                <label htmlFor="OffsetShadowBlur">Blurry</label>
                <input
                  id="OffsetShadowBlur"
                  type="range"
                  min="0"
                  max="50"
                  value={shadowBlurValue}
                  onChange={handleBlurOffset}
                  disabled={shadowIsEnabled ? false : true}
                  style={{
                    width: "100%",
                    appearance: shadowIsEnabled ? "" : "none", // Remove default styling
                    background: "#ddd", // Background color of the track
                    height: "5px", // Height of the track
                    borderRadius: "5px", // Round edges for the track
                    outline: "none", // Remove outline
                  }}
                />
                <div
                  style={{
                    top: "30px",
                    padding: "3px 7px",
                    borderRadius: "4px",
                    whiteSpace: "nowrap",
                    pointerEvents: "none",
                    fontSize: "small",
                  }}
                >
                  {shadowBlurValue}
                </div>
              </div>
              {/* Set Background Color  */}
              <div className="flex justify-between">
                <label htmlFor="setbgcolor">Background</label>
                <input
                  id="setbgcolor"
                  className="w-6 h-6 cursor-pointer rounded-color-input"
                  type="color"
                  value={BackgroundColor}
                  onChange={(e) => handleBGColor(e)}
                />
              </div>
            </div>
          </div>
        </div>
      )}
    </div>
  );
};

export default FontStrokeOptions;
